from jinja2 import FileSystemLoader, Environment
from pathlib import Path
import os

class Templates(object):
    def __init__(self, image:str, tag:str):
        self.image = image
        self.tag = tag
        self.jinja_extension = ".jinja"
        self.dockerfile_template_rpm = "DOCKERFILE-rpm.jinja"
        self.dockerfile_template_deb = "DOCKERFILE-deb.jinja"
        self.docker_compose_template = "docker-compose.yml.jinja"
        self.docker_directory = Path(__file__).parents[0] / "docker"
        self.templates_location = Path(__file__).parents[0] / "templates"
        self.rpm_distributions = ['centos']
        self.deb_distributions = ['debian','ubuntu']

    def print_file(self):
        print(self.docker_directory)

    def dockerfile_setup(self, packages:list):
        templateLoader = FileSystemLoader(searchpath=self.templates_location)
        templateEnv = Environment(loader=templateLoader)

        if self.image in self.rpm_distributions:
            dockerfile_template = self.dockerfile_template_rpm
        elif self.image in self.deb_distributions:
            dockerfile_template = self.dockerfile_template_deb
        else:
            raise NotImplemented(f"{self.image} is currently unsupported.")
        
        template = templateEnv.get_template(dockerfile_template)

        outputText = template.render(image=self.image,tag=self.tag,packages=packages)

        new_path = self.docker_directory / dockerfile_template.split(".jinja")[0]
        with open(new_path, 'w') as nf:
            nf.write(outputText)

    def base_packages_directory(self) -> Path:
        return self.docker_directory / Path(f"{self.image}_{self.tag}_packages")

    def volume_directory(self, packages:list) -> Path:
        base_package_dir = Path(f"{self.image}_{self.tag}_packages")
        for package in packages:
            os.makedirs(self.docker_directory / base_package_dir / package)
        return base_package_dir


from docker import from_env
from pathlib import Path
from pkgdownloader.templates import Templates

class Downloader(object):

    def __init__(self, os, version):
        self.docker_directory = Path(__file__).parents[0] / "docker"
        self.tag = f"packages_dowloader_{os}_{version}"
        self.client = from_env()
        self.os = os
        self.version = version
        self.rpm_distributions = ['centos','fedora']
        self.deb_distributions = ['debian','ubuntu']

    def build_image(self):

        if self.os in self.rpm_distributions:
            dockerfile_name = "DOCKERFILE-rpm"
        elif self.os in self.deb_distributions:
            dockerfile_name = "DOCKERFILE-deb"
        else:
            raise NotImplemented(f"{self.image} is currently unsupported.")

        output = self.client.images.build(
            path=str(self.docker_directory),
            dockerfile=dockerfile_name,
            tag=self.tag
        )

        return output

    def run_image(self, **kwargs):

        template = Templates(image=self.os,tag=self.version)

        if kwargs.get("location"):
            location =  Path(kwargs.get("location")) / Path(f"{self.os}_{self.version}_packages")
        else: 
            location = template.base_packages_directory()

        self.client.containers.run(
            image=self.tag,
            volumes={
                str(location): {
                    'bind': f'/{self.os}_{self.version}_packages',
                    'mode': 'rw'
                }
            },
            auto_remove=True
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pkgdownloader']

package_data = \
{'': ['*'], 'pkgdownloader': ['docker/.gitkeep', 'templates/*']}

install_requires = \
['Jinja2>=2.11.2,<3.0.0', 'docker>=4.3.1,<5.0.0']

entry_points = \
{'console_scripts': ['pkg-downloader = pkgdownloader.pkgdownloader:run']}

setup_kwargs = {
    'name': 'pkg-downloader',
    'version': '0.1.1',
    'description': 'A streamlined way to download packages for offline installation.',
    'long_description': '# pkg-downloader\n\nA package downloader for Ubuntu, Debian, and CentOS that downloads the .deb or .rpm packages and their dependencies.\n\n## Installation & Requirements\n---\n\n### Prerequisites\n\n- Python 3.7.2+\n- [Docker](https://docs.docker.com/engine/install/)\n\n\n### Install pkg-downloader\n```\npip3 install pkg-downloader\n```\n\n## Exaple Usage\n---\nTo download packages for centos 8: \n```\npkg-downloader --os centos --version 8 --packages nginx postgresql\n```\n\nTo download packages for ubuntu 20.04 to a specific directory: \n```\npkg-downloader --os ubuntu --version 20.04 --location ~/Desktop --packages nginx postgresql \n```',
    'author': 'Dan',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from pkgdownloader.downloader import Downloader
from argparse import ArgumentParser, Namespace
from pkgdownloader.templates import Templates
import os

argsparse = ArgumentParser()

def parser() -> Namespace:
    argsparse.add_argument(
        '--os',
        type=str,
        action='store',
        help="Target OS for which to pull packages.",
        dest="os_type"
        )

    argsparse.add_argument(
        '--version', 
        type=str,
        action='store',
        help="Target OS Version for which to pull packages.",
        dest="os_version"
        )

    argsparse.add_argument(
        '--packages', nargs="+",
        type=str,
        action='store',default=None,
        help="Packages to download.",
        dest="packages_list"
        )
    
    argsparse.add_argument(
        '--location',
        type=str,
        action='store',default=None,
        help="Target location for download.",
        dest="download_location"
        )

    args = argsparse.parse_args()

    return args

def buildTemplate(arguments):
    template = Templates(
        image=arguments.os_type,
        tag=arguments.os_version
    )

    template.dockerfile_setup(arguments.packages_list)

def dockerRunner(arguments):
    downloadManager = Downloader(
        os=arguments.os_type,
        version=arguments.os_version
    )

    downloadManager.build_image()
    download_location = arguments.download_location if arguments.download_location else os.getcwd()
    downloadManager.run_image(
        location=download_location
    )

def run():
    args = parser()
    buildTemplate(arguments=args)
    dockerRunner(arguments=args)
    print(f"[+] Finished downloading {args.packages_list}")